<?php 
namespace Concrete\Package\CcOptinFree\Controller\SinglePage\Dashboard\CcOptin;

use URL;
use Page;
use View;
use Loader;

use Concrete\Core\Page\Controller\DashboardPageController;
use Concrete\Core\Search\Column\Column as DatabaseItemListColumn;

use Concrete\Package\CcOptinFree\CcOptin;
use Concrete\Package\CcOptinFree\CcOptinDate;
use Concrete\Package\CcOptinFree\Models\CcOptinSubscriber;
use Concrete\Package\CcOptinFree\Models\CcOptinCampaign;
use Concrete\Package\CcOptinFree\Models\CcOptinDatabaseItemList;
use Concrete\Package\CcOptinFree\Controllers\CcOptinSecurity;
use Concrete\Package\CcOptinFree\Controllers\CcOptinBaseController;
use Concrete\Package\CcOptinFree\Controllers\CcOptinConcreteController;


class SubscriberListModel
{
	protected static $pageNames = array();

	protected $subscriber;
	private $urls;

	public function __construct($subscriberData) {
		$this->subscriber = $subscriberData;
		$this->urls = Loader::helper('concrete/urls');
	}

	///// Sync method names with SubscriberList query /////

	public function id() { return $this->subscriber['id']; }
	public function email() { return htmlentities($this->subscriber['email']); }
	public function created($format = null) { return CcOptinDate::fromMysqlUTC($this->subscriber['created'], $format); }
	public function ip() { return htmlentities($this->subscriber['ip']); }


}

class SubscriberList extends CcOptinDatabaseItemList
{
	private $campaignID;

	public function __construct($campaignID) {
		$SQL = 'SELECT id, email, firstName, lastName, phone, campaignID, ip, created FROM ' . CcOptinSubscriber::Table;
		$this->query = $SQL;

		$this->campaignID = 0;

		$columns = array(
			new DatabaseItemListColumn('email', t('Email'), 'email', true),
			new DatabaseItemListColumn('created', t('Subscribed'), 'created', true, 'desc')
		);

		parent::__construct($columns, 'created');
	}

	public function showingAll() { return $this->campaignID ? false : true; }

	public function name() {
		if($this->showingAll()) return t('All');

		return Page::getByID($this->campaignID)->getCollectionName();
	}

	public function get($itemsToGet = 0, $offset = 0) {
		return parent::get($itemsToGet, $offset, __NAMESPACE__ . '\SubscriberListModel');
	}
}

class Subscribers extends CcOptinConcreteController
{
	private $exportFormatKey = 'export.date_format';
	private $exportFormatDefault = 'Y-m-d H:i:s';

	private $exportSeparatorKey = 'export.separator';
	private $exportSeparatorDefault = 0;

	private $exportSeparators = array('comma', 'semicolon', 'tab');
	private $realSeparators = array(',', ';', "\t");

	protected $security;

	public function view() {
		if(!$this->security()->canRead()) {
			$v = View::getInstance();
			$v->render('/page_forbidden');
			exit;
		}

		$id = intval($_GET['id']);
		$list = new SubscriberList($id);
		$list->setItemsPerPage(50);

		$this->addAndRequireAsset('javascript', 'single_pages/dashboard/cc_optin/subscribers/view.js');
		$this->addAndRequireAsset('css', 'single_pages/dashboard/cc_optin/subscribers/view.css');
		$this->set('list', $list);

		$this->set('format', array(
			'format' => $this->getFormat(), 
			'defaultFormat' => $this->exportFormatDefault,
			'separator' => $this->getSeparator(),
			'separators' => $this->exportSeparators
		));

		$this->set('pageTitle', t('Subscribers') . ' - ' . $list->name());
	}

	public function action() {
		if(!$this->security()->canWrite())
			return $this->outputJsonError(t("You don't have write access."));

		switch($_POST['action']) {
			case 'delete': $this->_delete($_POST);
			case 'deleteall': $this->_deleteAll();
		}

		$this->outputJsonError(t('No valid action found.'));
	}

	private function _delete($post) {
		$list = $this->idList($post['id']);

		if(count($list)) {
			$listString = join(',', $list);
			$SQL = 'DELETE FROM ' . CcOptinSubscriber::Table . " WHERE id IN ($listString)";
			Loader::db()->execute($SQL);
		}

		$this->outputJsonSuccess(t2('%d subscriber was deleted.', '%d subscribers were deleted.', count($list), count($list)));
	}

	private function _deleteAll() {
		$SQL = 'DELETE FROM ' . CcOptinSubscriber::Table;
		Loader::db()->execute($SQL);

		$this->outputJsonSuccess(t('All subscribers were deleted.'));
	}	

	public function export() {
		if(!$this->security()->canRead()) {
			CcOptinBaseController::pageForbidden();
			return;
		}

		$th = Loader::helper("text");

		$id = intval($_GET['id']);
		$list = new SubscriberList($id);

		$format = $th->sanitize($_GET['format']);
		if(!$format) $format = $this->getFormat();

		$separator = $this->realSeparators[$_GET['separator']];
		if(!$separator) $separator = $this->realSeparators[$this->getSeparator()];

		ini_set('memory_limit', -1);
		set_time_limit(0);

		$dh = Loader::helper('date');
		$date = date('Y-m-d');

		$name = '';

		header("Content-Type: text/csv");
		header("Cache-control: private");
		header("Pragma: public");
		header("Content-Disposition: inline; filename=\"Opt-In subscribers{$name} {$date}.csv\""); 

		$fields = array('Email', 'Created', 'IP');
		foreach ($fields as $key => $value) $fields[$key] = '"'.$value.'"';
		echo join($separator, $fields) . "\r\n";

		foreach ($list->get() as $subscriber) {
			$created = $dh->date($format, strtotime($subscriber->created(CcOptinDate::dbFormat)));
			$data = array($subscriber->email(), $created, $subscriber->ip());
			foreach ($data as $key => $value) {
				$data[$key] = '"' . str_replace('"', '""', $value) . '"';
			}
			echo join($separator, $data) . "\r\n";
		}
		
		exit;
	}

	public function setFormat() {
		if(!$this->security()->canWrite())
			return $this->outputJsonError(t("You don't have write access."));

		$th = Loader::helper("text");
		$format = $th->sanitize($_POST['format']);
		$separator = intval($_POST['separator']);

		CcOptin::config()->save($this->exportFormatKey, $format);
		CcOptin::config()->save($this->exportSeparatorKey, $separator);

		$this->outputJsonSuccess();
	}

	private function getFormat() {
		$value = CcOptin::config()->get($this->exportFormatKey);
		return $value ? $value : $this->exportFormatDefault;
	}

	private function getSeparator() {
		$value = CcOptin::config()->get($this->exportSeparatorKey);
		return $value ? $value : $this->exportSeparatorDefault;
	}
}